@@ -12,6 +12,7 @@ import requests
 import logging
 import base64
 from hashlib import md5
+from deprecated import deprecated
 
 try:
     import simplejson as json
@@ -41,16 +42,18 @@ class PySendPulse:
     __token_hash_name = None
     __storage_type = "FILE"
     __refresh_token = 0
+    __memcached_host = "127.0.0.1:11211"
 
     MEMCACHED_VALUE_TIMEOUT = 3600
     ALLOWED_STORAGE_TYPES = ['FILE', 'MEMCACHED']
 
-    def __init__(self, user_id, secret, storage_type="FILE"):
+    def __init__(self, user_id, secret, storage_type="FILE", token_file_path="", memcached_host="127.0.0.1:11211"):
         """ SendPulse API constructor
 
         @param user_id: string REST API ID from SendPulse settings
         @param secret: string REST API Secret from SendPulse settings
         @param storage_type: string FILE|MEMCACHED
+        @param memcached_host: string Host for Memcached server, default is 127.0.0.1:11211
         @raise: Exception empty credentials or get token failed
         """
         logger.info("Initialization SendPulse REST API Class")
@@ -60,6 +63,8 @@ class PySendPulse:
         self.__user_id = user_id
         self.__secret = secret
         self.__storage_type = storage_type.upper()
+        self.__token_file_path = token_file_path
+        self.__memcached_host = memcached_host
         m = md5()
         m.update("{}::{}".format(user_id, secret).encode('utf-8'))
         self.__token_hash_name = m.hexdigest()
@@ -69,7 +74,7 @@ class PySendPulse:
             self.__storage_type = 'FILE'
         logger.debug("Try to get security token from '{}'".format(self.__storage_type, ))
         if self.__storage_type == "MEMCACHED":
-            mc = memcache.Client(['127.0.0.1:11211'])
+            mc = memcache.Client([self.__memcached_host])
             self.__token = mc.get(self.__token_hash_name)
         else:  # file
             filepath = "{}{}".format(self.__token_file_path, self.__token_hash_name)
@@ -102,11 +107,14 @@ class PySendPulse:
         logger.debug("Got: '{}'".format(self.__token, ))
         if self.__storage_type == "MEMCACHED":
             logger.debug("Try to set token '{}' into 'MEMCACHED'".format(self.__token, ))
-            mc = memcache.Client(['127.0.0.1:11211'])
+            mc = memcache.Client([self.__memcached_host])
             mc.set(self.__token_hash_name, self.__token, self.MEMCACHED_VALUE_TIMEOUT)
         else:
             filepath = "{}{}".format(self.__token_file_path, self.__token_hash_name)
             try:
+                if not os.path.isdir(self.__token_file_path):
+                    os.makedirs(self.__token_file_path, exist_ok=True)
+
                 with open(filepath, 'w') as f:
                     f.write(self.__token)
                     logger.debug("Set token '{}' into 'FILE' '{}'".format(self.__token, filepath))
@@ -160,7 +168,6 @@ class PySendPulse:
                 logger.debug("Request response: {}".format(response.json(), ))
             except:
                 logger.critical("Raw server response: {}".format(response.text, ))
-                return response.status_code
         return response
 
     def __handle_result(self, data):
@@ -969,9 +976,11 @@ class PySendPulse:
         logger.info("Function call: sms_add_phones_to_blacklist")
         return self.__handle_result(self.__send_request('sms/black_list', 'DELETE', data_to_send))
 
+    @deprecated(version='0.1.4', reason="You should use sms_add_campaign_by_addressbook_id")
     def sms_add_campaign(self, sender_name, addressbook_id, body, date=None, transliterate=False):
         """ Create new sms campaign
 
+        @deprecated: use method sms_delete_phonesfrom_blacklist
         @param sender_name: string senders name
         @param addressbook_id: unsigned int addressbook ID
         @param body: string campaign body
@@ -998,6 +1007,7 @@ class PySendPulse:
 
         return self.__handle_result(self.__send_request('sms/campaigns', 'POST', data_to_send))
 
+    @deprecated(version='0.1.4', reason="You should use sms_add_campaign_by_phones")
     def sms_send(self, sender_name, phones, body, date=None, transliterate=False):
         """ Send sms by some phones
 
@@ -1033,6 +1043,70 @@ class PySendPulse:
 
         return self.__handle_result(self.__send_request('sms/send', 'POST', data_to_send))
 
+    def sms_add_campaign_by_addressbook_id(self, sender_name, addressbook_id, body, additional_params={}):
+        """ Create new sms campaign by addressbook_id
+
+        @param sender_name: string senders name
+        @param addressbook_id: unsigned int addressbook ID
+        @param body: string campaign body
+        @param additional_params: dictionary additional params for sms task
+        @return: dictionary with response message
+        """
+
+        logger.info("Function call: sms_add_campaign_by_addressbook_id")
+        if not sender_name:
+            return self.__handle_error('Seems you not pass sender name')
+        if not addressbook_id:
+            return self.__handle_error('Seems you not pass addressbook ID')
+        if not body:
+            return self.__handle_error('Seems you not pass sms text')
+
+        data_to_send = {
+            'sender': sender_name,
+            'addressBookId': addressbook_id,
+            'body': body
+        }
+
+        if additional_params:
+            data_to_send.update(additional_params)
+
+        return self.__handle_result(self.__send_request('sms/campaigns', 'POST', data_to_send))
+
+    def sms_add_campaign_by_phones(self, sender_name, phones, body, additional_params={}):
+        """ Create new sms campaign by some phones
+
+        @param sender_name: string senders name
+        @param phones: array phones
+        @param body: string campaign body
+        @param additional_params: dictionary additional params for sms task
+        @return: dictionary with response message
+        """
+
+        logger.info("Function call: sms_add_campaign_by_phones")
+        if not sender_name:
+            return self.__handle_error('Seems you not pass sender name')
+        if not phones:
+            return self.__handle_error('Seems you not pass phones')
+        if not body:
+            return self.__handle_error('Seems you not pass sms text')
+
+        try:
+            phones = json.dumps(phones)
+        except:
+            logger.debug("Phones: {}".format(phones))
+            return self.__handle_error("Phones list can't be converted by JSON library")
+
+        data_to_send = {
+            'sender': sender_name,
+            'phones': phones,
+            'body': body,
+        }
+
+        if additional_params:
+            data_to_send.update(additional_params)
+
+        return self.__handle_result(self.__send_request('sms/send', 'POST', data_to_send))
+
     def sms_get_list_campaigns(self, date_from, date_to):
         """ SMS: get list of campaigns
 
@@ -1114,3 +1188,23 @@ class PySendPulse:
 
         logger.info("Function call: sms_delete_campaign")
         return self.__handle_result(self.__send_request('sms/campaigns', 'DELETE', data_to_send))
+
+    # ------------------------------------------------------------------ #
+    #                           EVENTS                                   #
+    # ------------------------------------------------------------------ #
+
+    def send_event(self, event_name, body):
+        """ Send event by slug
+
+        @param event_name: string event name
+        @param body: array body {'email': 'test@test.com', 'phone': '+123456789': 'var_1':'var_1_value'}
+        @return: dictionary with response message
+        """
+
+        logger.info("Function call: send_event")
+        if not event_name:
+            return self.__handle_error('Seems you not pass event slug')
+        if not body:
+            return self.__handle_error('Seems you not pass body')
+
+        return self.__handle_result(self.__send_request('/events/name/{}'.format(event_name, ), 'POST', body))
